# README.md

## Описание проекта

Данный проект представляет собой генератор строк на базе шаблонов и двух файлов конфигураций: `dates.ctl` и `periods.ctl`. Эти строки могут использоваться для различных целей, включая генерацию строк ориджина (Origin Line) в сети FidoNet. Генерация основана на динамическом вычислении времени, прошедшего или оставшегося до определённых событий, а также на расчёте прогресса временных периодов. Результат работы программы — файл(ы) со строками, сгенерированными по шаблонам, содержащим макросы для замены.

## Файлы конфигурации

### Файл `dates.ctl`

Этот файл описывает шаблоны для отображения строк с информацией о событиях, происходящих на определённые даты, и прогрессе относительно этих дат.

Формат строк в `dates.ctl`:

```
"<шаблон>", <дата>, <имя файла>
```

Пример записи в `dates.ctl`:

```
"Осень: [%PROGRESS1] %PERCENTS%", "░▓", 30, 01/09/0, 32/11/0, v-maker.lst
```

#### Макросы для `dates.ctl`

- `%DREMAIN` — сколько дней осталось до даты события.
- `%DPAST` — сколько дней прошло с даты события.
- `%MREMAIN` — сколько месяцев осталось до даты события.
- `%MPAST` — сколько месяцев прошло с даты события.
- `%HREMAIN` — сколько часов осталось до даты события.
- `%HPAST` — сколько часов прошло с даты события.
- `%DTOKEN` — склонение слова "день" в зависимости от значения `%DREMAIN` или `%DPAST`.
- `%MTOKEN` — склонение слова "месяц" в зависимости от значения `%MREMAIN` или `%MPAST`.
- `%HTOKEN` — склонение слова "час" в зависимости от значения `%HREMAIN` или `%HPAST`.
- `%QUOTE` — знак кавычки (`"`).

### Файл `periods.ctl`

Файл описывает временные периоды, для которых программа рассчитывает прогресс и оставшееся/прошедшее время.

Формат строк в `periods.ctl`:

```
"<шаблон>", <"символы фона и заливки">, <длина строки прогресса>, <дата начала периода>, <дата окончания периода>, <имя файла>
```

Пример записи в `periods.ctl`:

```
"Осень: [%PROGRESS1] %PERCENTS%", "░▓", 30, 01/09/0, 32/11/0, v-maker.lst
```

#### Макросы для `periods.ctl`

- `%PERCENTS` — процент завершённости периода.
- `%PROGRESS1` — строка прогресса в виде символов `заливки` и `фона` (например, `####------`).
- `%PROGRESS2` — альтернативный вид строки прогресса (например, `---#------`).
- `%DREMAIN` — сколько дней осталось до окончания периода.
- `%DPAST` — сколько дней прошло с начала периода.
- `%MREMAIN` — сколько месяцев осталось до окончания периода.
- `%MPAST` — сколько месяцев прошло с начала периода.
- `%HREMAIN` — сколько часов осталось до окончания периода.
- `%HPAST` — сколько часов прошло с начала периода.
- `%DPTOKEN`, `%DRTOKEN` — склонения слова "день" для `%DPAST` и `%DREMAIN`.
- `%MPTOKEN`, `%MRTOKEN` — склонения слова "месяц" для `%MPAST` и `%MREMAIN`.
- `%HPTOKEN`, `%HRTOKEN` — склонения слова "час" для `%HPAST` и `%HREMAIN`.
- `%QUOTE` — знак кавычки (`"`).

## Пример работы

На базе следующих конфигураций:

`dates.ctl`:

```
"Я топчу землю уже %DPAST %DTOKEN.", 10/04/1982, v-maker.lst
```

`periods.ctl`:

```
"Зима: [%PROGRESS1] %PERCENTS%", "░▓", 30, 01/12/-1, 32/02/0, v-maker.lst
"Весна: [%PROGRESS1] %PERCENTS%", "░▓", 30, 01/03/0, 32/05/0, v-maker.lst
"Лето: [%PROGRESS1] %PERCENTS%", "░▓", 30, 01/06/0, 32/08/0, v-maker.lst
"Осень: [%PROGRESS1] %PERCENTS%", "░▓", 30, 01/09/0, 32/11/0, v-maker.lst
"Месяц: [%PROGRESS1] %PERCENTS%", "-#", 60, 01/0/0, 32/0/0, v-maker.lst
"[1-е января] %PROGRESS2 [31-е декабря]", "─┼", 40, 01/01/0, 31/12/0, v-maker.lst
"Универ: [%PROGRESS1] %PERCENTS% закончен.", "░▓", 45, 01/11/2022, 100/06/2027, v-maker.lst
"Прошлый день рождения был %DPAST %DPTOKEN назад, следующий будет через %DREMAIN %DRTOKEN.", "░▓", 10, 10/04/-1, 10/04/0, v-maker.lst
"10-е число месяца наступит через %DREMAIN %DRTOKEN.", "  ", 0, 10/-1/0, 10/0/0, v-maker.lst
```

**6 октября 2024 года** будет сгенерирован файл `v-maker.lst` с такими строками:

```
Я топчу землю уже 15520 дней.
Осень: [▓▓▓▓▓▓▓▓▓▓▓▓░░░░░░░░░░░░░░░░░░] 39.56%
Месяц: [############------------------------------------------------] 19.35%
[1-е января] ──────────────────────────────┼───────── [31-е декабря]
Универ: [▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓░░░░░░░░░░░░░░░░░░░░░░░░░░] 41.46% закончен.
Прошлый день рождения был 179 дней назад, следующий будет через 186 дней.
10-е число месяца наступит через 4 дня.
```

Как видно выше, периоды, в которые мы не попали на момент генерации, были пропущены.

## Использование

1. Отредактируйте файлы `dates.ctl` и `periods.ctl` для своих целей.
2. Настройте регулярный запуск программы или запускайте её вручную когда нужно сгенерировать строки.
